//compilieren mit
// gcc -L/usr/local/lib  mqttCGIPub.c -lmosquitto -o mqttCGIPub.bin -L. -lurlCoding
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "mosquitto.h"
#include "urlcoding.h"

void on_publish(struct mosquitto *mosq, void *userdata, int mid){
  mosquitto_disconnect(mosq);
}

int main(int argc, char *argv[]){


  // das problem bei publish ist: wir muessen 2 werte von der webseite hochladen:
  // 1.: das topic
  // 2.: die zu veroeffentlichende message
  // das laesst sich mit hidden-fields im web-formular erreichen
  
  // folgender c-code ist eng mit den forumlaren auf der webseite verzahnt
  // der html-code der lichtschalter muss daher folgenden aufbau haben:

  /*
  	<form method="GET" action="http://mosquitto.wara.de/mqttCGIPub.bin">
	  <input type="hidden" name="topic" value="licht/garage"/>
	  <input type="hidden" name="message" value="an"/>
	  <input type="submit" value="an"/>
	</form>
  */

  //das ergibt einen QUERY_STRING der form (sichtbar in der url des browsers):
  //topic=licht%2Fgarage&message=an

  //1. schritt: rueckgaengig machen des URL-encodings:
  
  char encodiert[1024] = {0}; //string wird laenger, z.B. '#' -> %23
  char decodiert[256] = {0};
 
  // wir lesen die umgebungsvariable
  char* queryString = getenv("QUERY_STRING");
  // der QUERY_STRING ist url-encodiert, das machen wir rueckgaengig
  urlDecode(decodiert, queryString);

  // der decodierte QUERY_STRING sieht im beispiel dann so aus:
  // topic=licht/garage&message=an
  
  // d.h. der QUERY_STRING hat ein format wie z.b.:
  // topic=licht/garage&message=an

  // WENN wir voraussetzen, dass der QUERY_STRING immer genau so aufgebaut ist (auf webseite (s.o.) festlegen!)
  // DANN ist:
  // unser topic von den zeichen '=' und '&' begrenzt
  // der message geht immer ein '='-zeichen voran
  // somit koennen wir mit sscanf den QUERY_STRING zerlegen
  
  char topic[256] ;
  char message[256] ;

  // % steht fuer eine zeichensequenz
  // %* steht fuer eine zeichensequenz, die ignoriert wird
  // %c fuer ein einzelnes zeichen, %*c fuer ein einzelnes, zu ignorierendes zeichen
  // %[^=] steht fuer eine zeichensequenz, die kein = enthaelt
  // %s steht fuer eine zeichensequenz beliebiger laenge (also einen string)

  sscanf(decodiert, "%*[^=]%*c%[^&]%*[^=]%*c%s", topic, message);

  // erklaerung:
  // %*[^=] : wir lesen und ignorieren die zeichen, solange kein = kommt
  // %*c    : wir lesen und ignorieren ein einzelnes, weiteres zeichen (das '=')
  // %[^&]  : wir lesen die naechsten zeichen und speichern diese in 'topic' solange kein '&' kommt
  // %*[^=] : wir lesen und ignorieren die zeichen, solange kein = kommt
  // %*c    : wir lesen und ignorieren ein einzelnes, weiteres zeichen (das '=')
  // %s     : wir lesen die restlichen zeichen und speichern diese in  'message'

  //nun wird veroeffentlicht:

  struct mosquitto *mosq;
  
  mosq = mosquitto_new("id", true, NULL);
  mosquitto_publish_callback_set(mosq, on_publish);
  
  mosquitto_connect(mosq, "localhost", 1883, 0);
  mosquitto_publish(mosq, NULL, topic, 3, message, 0, true);
    
  mosquitto_loop_forever(mosq, -1, 1);  // this calls mosquitto_loop() in a loop, it will exit once the client disconnects cleanly

  //redirect sofort zurueck zur html-seite
  printf("Location: http://mosquitto.wara.de\n\n");
  
  //testausgabe auf webseite
  /*printf("Content-Type: text/html\n\n");
  printf("<html>\n<body>\n");

  printf("query_string: %s\n", decodiert);

  printf("topic: %s\n", topic);
  printf("message: %s\n", message);
  
  printf("<h1>CGI: publish</h1>\n");
  printf("<p>das cgi-programm sendet eine Meldung an den \n");
  printf("mqtt-broker. das thema (topic) und die meldung (message) werden mit hidden-fields in einem webformular gesetzt.</p>\n");

  
  printf("</body></html>");
  */
  
  return 0;
}


