//compilieren mit
// gcc -L/usr/local/lib  single.c -lmosquitto -o single.bin
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "mosquitto.h"

// zwei tabellen fuer das url-encoding
char rfc3986[256] = {0};
char html5[256] = {0};

//funktion, die url-encoding erledigt
char* urlEncode( char *table, unsigned char *s, char *enc){

  for (; *s; s++){

    if (table[*s]) sprintf( enc, "%c", table[*s]);
    else sprintf( enc, "%%%02X", *s);
    while (*++enc);
  }

  return( enc);
}

//funktion fuer das url-decoding
void urlDecode(char *dst, const char *src)
{
  char a, b;
  while (*src) {
    if ((*src == '%') &&
	((a = src[1]) && (b = src[2])) &&
	(isxdigit(a) && isxdigit(b))) {
      if (a >= 'a')
	a -= 'a'-'A';
      if (a >= 'A')
	a -= ('A' - 10);
      else
	a -= '0';
      if (b >= 'a')
	b -= 'a'-'A';
      if (b >= 'A')
	b -= ('A' - 10);
      else
	b -= '0';
      *dst++ = 16*a+b;
      src+=3;
    } else if (*src == '+') {
      *dst++ = ' ';
      src++;
    } else {
      *dst++ = *src++;
    }
  }
  *dst++ = '\0';
}

void urlEncoderRfcTablesInit(){
  int i;

  for (i = 0; i < 256; i++){

    rfc3986[i] = isalnum( i) || i == '~' || i == '-' || i == '.' || i == '_' ? i : 0;
    html5[i] = isalnum( i) || i == '*' || i == '-' || i == '.' || i == '_' ? i : (i == ' ') ? '+' : 0;
  }
}


int main(int argc, char *argv[])
{

  char encodiert[1024] = {0}; //string wird laenger, z.B. '#' -> %23
  char decodiert[256] = {0};

  /*
  urlEncoderRfcTablesInit();
  urlEncode(rfc3986, "+/#", encodiert);
  printf("url-codiert: %s\n", encodiert);

  urlDecode(decodiert, encodiert);
  printf("und wieder zurueck: %s\n", decodiert);
  */
  
  //wir lesen die umgebungsvariable
  char* queryString = getenv("QUERY_STRING");
  //der QUERY_STRING ist url-encodiert, das machen wir rueckgaengig
  urlDecode(decodiert, queryString);

  //jetzt wird der QUERY_STRING in ein name-werte-paar zerlegt
  char nameBuf[256] ;
  char wertBuf[256] ;

  // % steht fuer eine zeichensequenz
  // %* steht fuer eine zeichensequenz, die ignoriert wird
  // %c fuer ein einzelnes zeichen, %*c fuer ein einzelnes, zu ignorierendes zeichen
  // %[^=] steh fuer eine zeichensequenz, die kein = enthaelt
  // %s steht fuer eine zeichensequenz beliebiger laenge (also einen string)

  //sscanf(queryString, "%[^=]%*c%s", nameBuf, wertBuf);
  sscanf(decodiert, "%[^=]%*c%s", nameBuf, wertBuf);

  printf("testausgabe: %s\n", wertBuf);
  
  printf("Content-Type: text/html\n\n");
  printf("<html>\n<body>\n");
  
  int rc;
  struct mosquitto_message *msg;

  mosquitto_lib_init();

  rc = mosquitto_subscribe_simple(
				  &msg, 1, true,
				  //"test/#", 0,
				  wertBuf, 0,
				  "localhost", 1883,
				  NULL, 60, true,
				  NULL, NULL,
				  NULL, NULL);

  if(rc){
    printf("Error: %s\n", mosquitto_strerror(rc));
    mosquitto_lib_cleanup();
    return rc;
  }

  //printf("%s %s\n", msg->topic, (char *)msg->payload);
  printf("<h1>Testprogramm: libmosquitto + apache2</h1>\n");
  printf("<p>das cgi-programm holt sich eine meldung vom \n");
  printf("mqtt-broker. das thema (topic) ist test/#</p>\n");
  printf ("<p> damit es keinen http-timeout gibt, muss der \n");
  printf("broker eine retained-message bereithalten.</p>\n");
  printf("<p>Topic: %s</p> <p>Message: %s</p>\n", msg->topic, (char *)msg->payload);
  //
  printf("<h1>Test des URL-En-und-Decodings</h1>\n");
  printf("<p>QUERY_STRING: %s</p>\n", queryString);
  printf("<p>QUERY_STRING decodiert: %s</p>\n", decodiert);
  //
  printf("<h1>QUERY_STRING zerlegt in name/wert-Paar</h1>\n");
  printf("<p>parameter-name: %s</p>\n", nameBuf);
  printf("<p>parameter-wert: %s</p>\n", wertBuf);
  mosquitto_message_free(&msg);

  mosquitto_lib_cleanup();

  printf("</body></html>");
  
  return 0;
}


